@@ -1,6 +1,7 @@
-;;; navi2ch-board-misc.el --- Miscellaneous Functions for Navi2ch Board Mode
+;;; navi2ch-board-misc.el --- Miscellaneous Functions for Navi2ch Board Mode -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2004 by Navi2ch Project
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2008 by Navi2ch
+;; Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: 2ch, network
@@ -20,12 +21,20 @@
 ;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
+;;; Commentary:
+
+;;
+
 ;;; Code:
 (provide 'navi2ch-board-misc)
 (defconst navi2ch-board-misc-ident
-  "$Id: navi2ch-board-misc.el,v 1.92 2004/05/02 14:41:51 nanashi Exp $")
+  "$Id$")
 
-(eval-when-compile (require 'cl))
+(eval-when-compile 
+  (require 'cl)
+  (defvar navi2ch-board-last-seen-alist)
+  (defvar navi2ch-board-subject-alist)
+  (defvar navi2ch-board-current-board))
 
 ;; Avoid byte-compile warnings (contrib/izonmoji-mode.el).
 (eval-when-compile (defvar izonmoji-mode nil))
@@ -61,6 +70,7 @@
     ;; mark command
     (define-key map "*" 'navi2ch-bm-mark)
     (define-key map "u" 'navi2ch-bm-unmark)
+    (define-key map "m" nil)
     (define-key map "mr" 'navi2ch-bm-mark-region)
     (define-key map "ma" 'navi2ch-bm-mark-all)
     (define-key map "mA" 'navi2ch-bm-add-global-bookmark-mark-article)
@@ -104,38 +114,52 @@
 (defvar navi2ch-bm-fetched-article-list nil)
 (defvar navi2ch-bm-board-type-alist nil)
 
-(defvar navi2ch-bm-state-alist
-  '((view "V"
-	  navi2ch-bm-view-face
-	  navi2ch-bm-new-view-face
-	  navi2ch-bm-updated-view-face
-	  navi2ch-bm-seen-view-face)
-    (cache "C"
-	   navi2ch-bm-cache-face
-	   navi2ch-bm-new-cache-face
-	   navi2ch-bm-updated-cache-face
-	   navi2ch-bm-seen-cache-face)
-    (update "U"
-	    navi2ch-bm-update-face
-	    navi2ch-bm-new-update-face
-	    navi2ch-bm-updated-update-face
-	    navi2ch-bm-seen-update-face)
-    (nil " "
-	 navi2ch-bm-unread-face
-	 navi2ch-bm-new-unread-face
-	 navi2ch-bm-updated-unread-face
-	 navi2ch-bm-seen-unread-face)
-    (mark " "
-	  navi2ch-bm-mark-face
-	  navi2ch-bm-new-mark-face
-	  navi2ch-bm-updated-mark-face
-	  navi2ch-bm-seen-mark-face)))
+(defvar navi2ch-bm-state-char-table
+  (navi2ch-alist-to-hash
+   '((view   . ?V)
+     (cache  . ?C)
+     (update . ?U)
+     (nil    . ? ))
+   :test 'eq))
+  
 
-(defvar navi2ch-bm-updated-mark-alist
-  '((new . "%")
-    (updated . "+")
-    (seen . "=")
-    (nil . " ")))
+(eval-and-compile
+  (let ((state-list '(view cache update nil))
+	(update-list '(nil new updated seen)))
+    (let ((func (lambda (f)
+		  (navi2ch-alist-to-hash
+		   (mapcar (lambda (state)
+			     (cons state
+				   (navi2ch-alist-to-hash
+				    (mapcar (lambda (update)
+					      (cons update
+						    (funcall f state update)))
+					    update-list)
+				    :test 'eq)))
+			   state-list)
+		   :test 'eq))))
+      (defconst navi2ch-bm-state-face-table
+	(funcall func
+		 (lambda (state update)
+		   (intern (format "navi2ch-bm%s-%s-face"
+				   (if update
+				       (format "-%s" update)
+				     "")
+				   (or state 'unread))))))
+      (defconst navi2ch-bm-state-mark-face-table
+	(funcall func
+		 (lambda (state update)
+		   (intern (format "navi2ch-bm%s-mark-face"
+				   (if update
+				       (format "-%s" update)
+				     "")))))))))
+
+(defconst navi2ch-bm-updated-mark-table
+  (navi2ch-alist-to-hash '((new     . ?%)
+			   (updated . ?+)
+			   (seen    . ?=)
+			   (nil     . ? ))
+			 :test 'eq))
 
 (defvar navi2ch-bm-move-downward t)
 
@@ -192,19 +216,23 @@
   (run-hooks 'navi2ch-bm-select-board-hook)
   (navi2ch-set-mode-line-identification))
 
-(defsubst navi2ch-bm-set-property (begin end item state &optional updated)
+(defun navi2ch-bm-set-property (begin end item state &optional updated mark)
   (navi2ch-bm-set-property-internal begin end item)
-  (setq updated (or updated (get-text-property (1+ begin) 'updated)))
-  (put-text-property begin end 'updated updated)
-  (put-text-property begin end 'mouse-face navi2ch-bm-mouse-face)
-  (put-text-property begin end 'face (nth (cond ((null updated) 2)
-						((eq updated 'new) 3)
-						((eq updated 'updated) 4)
-						((eq updated 'seen) 5))
-					  (assq state
-						navi2ch-bm-state-alist))))
+  (let ((updated (or updated 
+		     (get-text-property begin 'navi2ch-bm-updated)))
+	(face-table (if mark
+			navi2ch-bm-state-mark-face-table
+		      navi2ch-bm-state-face-table)))
+    (add-text-properties begin end
+			 (list 'navi2ch-bm-updated updated
+			       'navi2ch-bm-state state
+			       'navi2ch-bm-mark mark
+			       'mouse-face navi2ch-bm-mouse-face
+			       'face
+			       (gethash updated
+					(gethash state face-table))))))
 
-(defsubst navi2ch-bm-get-state-from-article (board article)
+(defun navi2ch-bm-get-state-from-article (board article)
   (cond ((navi2ch-board-from-file-p board)
 	 (cond ((get-buffer (navi2ch-article-get-buffer-name
 			     board article))
@@ -217,25 +245,29 @@
 	(t
 	 (navi2ch-article-check-cached board article))))
 
-(defsubst navi2ch-bm-insert-subject (item number subject other
-					  &optional updated)
+(defun navi2ch-bm-format-subject
+  (number updated-char state-char subject other)
+  (format (concat "%" (number-to-string navi2ch-bm-number-width)
+		  "d %c%c %s%s%s\n")
+	  number updated-char state-char subject
+	  (make-string (max (- navi2ch-bm-subject-width
+			       (string-width subject))
+			    1)
+		       ? )
+	  other))
+
+(defun navi2ch-bm-insert-subject (item number subject other
+				       &optional updated)
   (let* ((article (navi2ch-bm-get-article-internal item))
 	 (board (navi2ch-bm-get-board-internal item))
 	 (point (point))
 	 (state (navi2ch-bm-get-state-from-article board article))
-	 string)
-    (unless subject (setq subject navi2ch-bm-empty-subject))
-    (setq string (format (concat "%" (number-to-string navi2ch-bm-number-width)
-				 "d %s%s %s%s%s\n")
-			 number
-			 (cdr (assq updated navi2ch-bm-updated-mark-alist))
-			 (cadr (assq state navi2ch-bm-state-alist))
-			 subject
-			 (make-string (max (- navi2ch-bm-subject-width
-					      (string-width subject))
-					   1)
-				      ? )
-			 other))
+	 (string (navi2ch-bm-format-subject
+		  number
+		  (gethash updated navi2ch-bm-updated-mark-table)
+		  (gethash state navi2ch-bm-state-char-table)
+		  (or subject navi2ch-bm-empty-subject)
+		  other)))
     ;; for contrib/izonmoji-mode.el
     (navi2ch-ifxemacs
 	(insert string)
@@ -270,16 +302,16 @@
 	(navi2ch-list)))))
 
 ;;; goto-*-column
-(defun navi2ch-bm-goto-updated-mark-column ()
+(defsubst navi2ch-bm-goto-updated-mark-column ()
   (beginning-of-line)
   (when (looking-at " *[0-9]+ ")
     (goto-char (match-end 0))))
 
-(defun navi2ch-bm-goto-state-column ()
+(defsubst navi2ch-bm-goto-state-column ()
   (when (navi2ch-bm-goto-updated-mark-column)
     (forward-char 1)))
 
-(defun navi2ch-bm-goto-mark-column ()
+(defsubst navi2ch-bm-goto-mark-column ()
   (when (navi2ch-bm-goto-updated-mark-column)
     (forward-char 2)))
 
@@ -304,29 +336,19 @@
       (navi2ch-bm-goto-state-column)
       (backward-char 1)
       (delete-char 2)
-      (insert (cdr (assq updated navi2ch-bm-updated-mark-alist)))
-      (insert (cadr (assq state navi2ch-bm-state-alist)))
+      (insert (gethash updated navi2ch-bm-updated-mark-table)
+	      (gethash state navi2ch-bm-state-char-table))
       (navi2ch-bm-set-property (navi2ch-line-beginning-position)
 			       (navi2ch-line-end-position)
 			       item state updated))))
 
-(defun navi2ch-bm-get-state (&optional point)
-  "$B$=$N0LCV$N(B state $B$rD4$Y$k(B"
-  (save-excursion
-    (and point (goto-char point))
-    (navi2ch-bm-goto-state-column)
-    (cdr (assoc (char-to-string (char-after))
-		(mapcar (lambda (x)
-			  (cons (cadr x) (car x)))
-			navi2ch-bm-state-alist)))))
+(defsubst navi2ch-bm-get-state (&optional point)
+  "$B$=$N0LCV$N(B state $B$rD4$Y$k!#(B"
+  (get-text-property (or point (point)) 'navi2ch-bm-state))
 
-(defun navi2ch-bm-get-updated-mark (&optional point)
-  "$B$=$N0LCV$N(B updated-mark $B$rD4$Y$k(B"
-  (save-excursion
-    (and point (goto-char point))
-    (when (navi2ch-bm-goto-updated-mark-column)
-      (car (rassoc (char-to-string (char-after))
-		   navi2ch-bm-updated-mark-alist)))))
+(defsubst navi2ch-bm-get-updated-mark (&optional point)
+  "$B$=$N0LCV$N(B updated-mark $B$rD4$Y$k!#(B"
+  (get-text-property (or point (point)) 'navi2ch-bm-updated))
 
 (defun navi2ch-bm-select-article (&optional max-line)
   (interactive "P")
@@ -360,7 +382,7 @@
       (set-window-configuration window-configuration))))
 
 (defun navi2ch-bm-show-url ()
-  "$BHD$N(B url $B$rI=<($7$F!"$=$N(B url $B$r8+$k$+(B kill ring $B$K%3%T!<$9$k(B"
+  "$BHD$N(B url $B$rI=<($7$F!"$=$N(B url $B$r8+$k$+(B kill ring $B$K%3%T!<$9$k!#(B"
   (interactive)
   (let* ((board (navi2ch-bm-get-board-internal
 		 (navi2ch-bm-get-property-internal (point))))
@@ -370,19 +392,20 @@
       (let ((char (navi2ch-read-char-with-retry
 		   (format "c)opy v)iew t)itle? URL: %s: " url)
 		   nil '(?c ?v ?t))))
-	(if (eq char ?t) (navi2ch-bm-copy-title board)
-	  (funcall (cond ((eq char ?c)
-			  (lambda (x)
-			    (kill-new x)
-			    (message "copy: %s" x)))
-			 ((eq char ?v)
-			  (lambda (x)
-			    (navi2ch-browse-url-internal x)
-			    (message "view: %s" x))))
-		   (navi2ch-bm-show-url-subr board)))))))
+	(if (eq char ?t)
+	    (navi2ch-bm-copy-title board)
+	  (setq url (navi2ch-bm-show-url-subr board))
+	  (cond ((not url)
+		 (message "Can't select this line!"))
+		((eq char ?c)
+		 (kill-new url)
+		 (message "Copy: %s" url))
+		((eq char ?v)
+		 (navi2ch-browse-url-internal url)
+		 (message "View: %s" url))))))))
 
 (defun navi2ch-bm-show-url-subr (board)
-  "$B%a%K%e!<$rI=<($7$F!"(Burl $B$rF@$k(B"
+  "$B%a%K%e!<$rI=<($7$F!"(Burl $B$rF@$k!#(B"
   (let ((char (navi2ch-read-char-with-retry
 	       (format "b)oard a)rticle l)ast%d: "
 		       navi2ch-article-show-url-number)
@@ -390,12 +413,14 @@
 	(article (navi2ch-bm-get-article-internal
 		  (navi2ch-bm-get-property-internal (point)))))
     (cond ((eq char ?b) (navi2ch-board-to-url board))
-	  ((eq char ?a) (navi2ch-article-to-url board article))
+	  ((eq char ?a) (when article
+			  (navi2ch-article-to-url board article)))
 	  ((eq char ?l) (let ((l (format "l%d" navi2ch-article-show-url-number)))
-			  (navi2ch-article-to-url board article l l))))))
+			  (when article
+			    (navi2ch-article-to-url board article l l)))))))
 
 (defun navi2ch-bm-copy-title (board)
-  "$B%a%K%e!<$rI=<($7$F!"%?%$%H%k$rF@$k(B"
+  "$B%a%K%e!<$rI=<($7$F!"%?%$%H%k$rF@$k!#(B"
   (navi2ch-article-copy-title board
 			      (navi2ch-bm-get-article-internal
 			       (navi2ch-bm-get-property-internal
@@ -556,7 +581,7 @@
           (setq i (1+ i)))))))
 
 (defun navi2ch-bm-view-logo ()
-  "$B$=$NHD$N%m%4$r8+$k(B"
+  "$B$=$NHD$N%m%4$r8+$k!#(B"
   (interactive)
   (let ((board (navi2ch-bm-get-board-internal
 		(navi2ch-bm-get-property-internal (point))))
@@ -584,7 +609,7 @@
       (message "Can't find logo file"))))
 
 (defun navi2ch-bm-add-global-bookmark (&optional bookmark-id)
-  (interactive (list (navi2ch-bookmark-read-id "bookmark id: ")))
+  (interactive (list (navi2ch-bookmark-read-id "Bookmark ID: ")))
   (let* ((item (navi2ch-bm-get-property-internal (point)))
 	 (board (navi2ch-bm-get-board-internal item))
 	 (article (navi2ch-bm-get-article-internal item)))
@@ -622,26 +647,18 @@
 INTERACTIVE $B$,(B non-nil $B$J$i(B mark $B$7$?$"$H0\F0$9$k!#(B
 ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
   (let ((item (navi2ch-bm-get-property-internal (point)))
-	(state 'mark)
-	(alist (mapcar
-		(lambda (x)
-		  (cons (cadr x) (car x)))
-		navi2ch-bm-state-alist)))
+	(state (navi2ch-bm-get-state (point)))
+	(table (and mark navi2ch-bm-state-mark-face-table)))
     (when item
-      (let ((buffer-read-only nil))
-	(when (string= mark " ")
-	  (save-excursion
-	    (navi2ch-bm-goto-state-column)
-	    (setq state (cdr (assoc (char-to-string (char-after (point)))
-				    alist)))))
-	(let ((point (point)))
-	  (navi2ch-bm-goto-mark-column)
-	  (delete-char 1)
-	  (insert mark)
-	  (navi2ch-bm-set-property (navi2ch-line-beginning-position)
-				   (navi2ch-line-end-position)
-				   item state)
-	  (goto-char point))))
+      (let ((buffer-read-only nil)
+	    (pos (point)))
+	(navi2ch-bm-goto-mark-column)
+	(delete-char 1)
+	(insert (if mark ?* ? ))
+	(navi2ch-bm-set-property (navi2ch-line-beginning-position)
+				 (navi2ch-line-end-position)
+				 item state nil table)
+	(goto-char pos)))
     (when (and navi2ch-bm-mark-and-move interactive)
       (let (downward)
 	(cond ((eq navi2ch-bm-mark-and-move 'follow)
@@ -655,11 +672,11 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
 
 (defun navi2ch-bm-mark (&optional arg)
   (interactive "P")
-  (navi2ch-bm-mark-subr "*" arg (interactive-p)))
+  (navi2ch-bm-mark-subr t arg (interactive-p)))
 
 (defun navi2ch-bm-unmark (&optional arg)
   (interactive "P")
-  (navi2ch-bm-mark-subr " " arg (interactive-p)))
+  (navi2ch-bm-mark-subr nil arg (interactive-p)))
 
 (defun navi2ch-bm-exec-subr (func &rest args)
   (save-excursion
@@ -677,16 +694,17 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
 	    (discard-input))
 	(forward-line)))))
 
-(defun navi2ch-bm-display-mark-article ()
+(defsubst navi2ch-bm-display-mark-article ()
   (interactive)
   (navi2ch-bm-exec-subr 'navi2ch-bm-display-article))
 
 (defun navi2ch-bm-fetch-mark-article (&optional force)
   (interactive "P")
-  (navi2ch-bm-exec-subr 'navi2ch-bm-fetch-article force))
+  (unless navi2ch-offline
+    (navi2ch-bm-exec-subr #'navi2ch-bm-fetch-article force)))
 
 (defun navi2ch-bm-textize-mark-article (directory &optional file)
-  (interactive "Ddirectory: \nFlist file: ")
+  (interactive "DDirectory: \nFList file: ")
   (let ((buffer (get-buffer-create (make-temp-name "*navi2ch "))))
     (navi2ch-bm-exec-subr 'navi2ch-bm-textize-article directory buffer)
     (save-excursion
@@ -696,7 +714,7 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
     (kill-buffer buffer)))
 
 (defun navi2ch-bm-add-global-bookmark-mark-article (bookmark-id)
-  (interactive (list (navi2ch-bookmark-read-id "bookmark id: ")))
+  (interactive (list (navi2ch-bookmark-read-id "Bookmark ID: ")))
   (navi2ch-bm-exec-subr 'navi2ch-bm-add-global-bookmark bookmark-id))
 
 
@@ -723,14 +741,15 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
 							       (point-min)))
 					       (end-of-line)
 					       (point))
-			       (if arg " " "*")))
+			       (not arg)))
 
 (defun navi2ch-bm-fetch-maybe-new-articles ()
   "$B99?7$5$l$F$$$k2DG=@-$N$"$k%9%l$r(B fetch $B$9$k!#(B"
   (interactive)
-  (navi2ch-bm-mark-states "[^=]")
-  (sit-for 0)
-  (navi2ch-bookmark-fetch-mark-article))
+  (unless navi2ch-offline
+    (navi2ch-bm-mark-states "[^=]")
+    (sit-for 0)
+    (navi2ch-bookmark-fetch-mark-article)))
 
 (defun navi2ch-bm-mark-all (&optional arg)
   (interactive "P")
@@ -748,16 +767,16 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
     (while (not (eobp))
       (navi2ch-bm-goto-updated-mark-column)
       (when (looking-at regexp)
-	(navi2ch-bm-mark-subr (if arg " " "*")))
+	(navi2ch-bm-mark-subr (not arg)))
       (forward-line))))
 
 ;; mark by regexp query
 (defun navi2ch-bm-mark-by-query (query &optional arg)
-  (interactive "Mquery (regexp): ")
+  (interactive "MQuery (regexp): ")
   (save-excursion
     (goto-char (point-min))
     (while (re-search-forward query nil t)
-      (navi2ch-bm-mark-subr (if arg " " "*")))))
+      (navi2ch-bm-mark-subr (not arg)))))
 
 ;;; sort
 (defun navi2ch-bm-sort-subr (rev start-key-fun end-key-fun)
@@ -771,8 +790,15 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
   (interactive "P")
   (navi2ch-bm-sort-subr
    rev
-   'beginning-of-line
-   'navi2ch-bm-goto-state-column))
+   (lambda ()
+     (beginning-of-line)
+     (save-match-data
+       (if (looking-at "^ *\\([0-9]+\\)")
+	   (string-to-number
+	    (buffer-substring (match-beginning 1) (match-end 1)))
+	 ;; not a number
+	 -1)))
+   nil))
 
 (defun navi2ch-bm-sort-by-state (&optional rev)
   (interactive "P")
@@ -906,8 +932,7 @@ ARG $B$,(B non-nil $B$J$i0\F0J}8~$r5U$K$9$k!#(B"
 				   (navi2ch-bm-get-state-from-article
 				    board article)))
 			(updated (or updated
-				     (get-text-property (point)
-							'updated))))
+				     (navi2ch-bm-get-updated-mark))))
 		    (navi2ch-bm-insert-state item state updated)
 		    (navi2ch-bm-set-property (navi2ch-line-beginning-position)
 					     (navi2ch-line-end-position)
@@ -928,18 +953,22 @@ ARTILCES $B$,(B alist $B$N>l9g$O$=$N%9%l$N$_$r!"(Balist $B$N(B list $B$N>
       (let ((artid (cdr (assq 'artid article)))
 	    (buffer (get-buffer (navi2ch-article-get-buffer-name board
 								 article)))
+	    (info-file (navi2ch-article-get-info-file-name board article))
 	    elt)
 	(when buffer
 	  (delete-windows-on buffer)
 	  (kill-buffer buffer))
-	(dolist (file (list (navi2ch-article-get-info-file-name board article)
+	(dolist (file (list info-file
+			    (navi2ch-make-backup-file-name
+			     info-file)
 			    (navi2ch-article-get-file-name board article)
 			    (navi2ch-article-get-message-filter-cache-file-name
 			     board article)))
 	  (condition-case nil
 	      (if (file-exists-p file)
 		  (delete-file file))
-	    (file-error nil)))
+	    (file-error nil))
+	  (navi2ch-cache-remove file navi2ch-info-cache))
 	(navi2ch-bm-remove-fetched-article board article)
 	(while (setq elt (assoc artid summary))
 	  (setq summary (delq elt summary))))
