@@ -1,6 +1,7 @@
-;;; navi2ch-bookmark.el --- global bookmark module for navi2ch
+;;; navi2ch-bookmark.el --- global bookmark module for navi2ch -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001-2004 by Navi2ch Project
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2008 by Navi2ch
+;; Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -27,7 +28,7 @@
 ;;; Code:
 (provide 'navi2ch-bookmark)
 (defconst navi2ch-bookmark-ident
-  "$Id: navi2ch-bookmark.el,v 1.39 2004/05/02 15:12:50 nanashi Exp $")
+  "$Id$")
 
 (eval-when-compile (require 'cl))
 
@@ -47,6 +48,7 @@
     (define-key map "O" 'navi2ch-bookmark-copy)
     (define-key map "mi" 'navi2ch-bookmark-fetch-mark-article)
     (define-key map "md" 'navi2ch-bookmark-cut-mark-article)
+    (define-key map "mD" 'navi2ch-bookmark-delete-mark-article)
     (define-key map "mo" 'navi2ch-bookmark-move-mark-article)
     (define-key map "s" 'navi2ch-bookmark-sync)
     (define-key map "X" 'navi2ch-bookmark-remember-order)
@@ -59,41 +61,43 @@
    nil))
 
 (defvar navi2ch-bookmark-list nil
-  "bookmark $B$rI=$9%j%9%H(B
+  "bookmark $B$rI=$9%j%9%H!#(B
 \((BOOKMARK-ID BOOKMARK-NAME
   (KEY
    (board BOARD)
    (article ARTICLE))...)
  ...)
 $B$H$$$&7A$r$7$F$$$k!#(B
-KEY $B$O(B (concat URI ARTID) ")
+KEY $B$O(B (concat URI ARTID)")
 (defvar navi2ch-bookmark-cut-stack nil)
 (defvar navi2ch-bookmark-current-bookmark-id nil)
+(defvar navi2ch-bookmark-fetch-mark-article-check-update t)
+(defvar navi2ch-bookmark-fetch-mark-article-no-check-regexp nil)
 
 ;;; navi2ch-bm callbacks
 (defun navi2ch-bookmark-set-property (begin end item)
   (put-text-property begin end 'item item))
 
-(defun navi2ch-bookmark-get-property (point)
+(defsubst navi2ch-bookmark-get-property (point)
   (get-text-property
    (save-excursion (goto-char point)
 		   (beginning-of-line)
 		   (point))
    'item))
 
-(defun navi2ch-bookmark-get-article (item)
+(defsubst navi2ch-bookmark-get-article (item)
   (cdr (assq 'article
 	     (cdr (assoc item
 			 (cddr (assoc navi2ch-bookmark-current-bookmark-id
 				      navi2ch-bookmark-list)))))))
 
-(defun navi2ch-bookmark-get-board (item)
+(defsubst navi2ch-bookmark-get-board (item)
   (cdr (assq 'board
 	     (cdr (assoc item
 			 (cddr (assoc navi2ch-bookmark-current-bookmark-id
 				      navi2ch-bookmark-list)))))))
 
-(defun navi2ch-bookmark-exit ()
+(defsubst navi2ch-bookmark-exit ()
   (run-hooks 'navi2ch-bookmark-exit-hook))
 
 ;; regist board
@@ -140,7 +144,7 @@ KEY $B$O(B (concat URI ARTID) ")
 	(throw 'exist t)))))
 
 (defun navi2ch-bookmark-create-bookmark (bookmark-id)
-  "$B%V%C%/%^!<%/(B BOOKMARK-ID $B$rDI2C$9$k(B"
+  "$B%V%C%/%^!<%/(B BOOKMARK-ID $B$rDI2C$9$k!#(B"
   (unless (assoc bookmark-id navi2ch-bookmark-list)
     (when (member bookmark-id
 		  (mapcar (lambda (x) (cdr (assq 'id x)))
@@ -157,18 +161,18 @@ KEY $B$O(B (concat URI ARTID) ")
       (navi2ch-bookmark-save-info))))
 
 (defun navi2ch-bookmark-delete-bookmark (bookmark-id)
-  "$B%V%C%/%^!<%/(B BOOKMARK-ID $B$r:o=|$9$k(B"
+  "$B%V%C%/%^!<%/(B BOOKMARK-ID $B$r:o=|$9$k!#(B"
   (let ((bookmark (assoc bookmark-id navi2ch-bookmark-list)))
     (when (y-or-n-p (concat "delete global bookmark "
 			    (cadr bookmark) "? "))
       (setq navi2ch-bookmark-list (delete bookmark navi2ch-bookmark-list))
-      (save-excursion
+      (save-current-buffer
 	(set-buffer navi2ch-list-buffer-name)
 	(navi2ch-list-sync-global-bookmark-category))
       (navi2ch-bookmark-save-info))))
 
 (defun navi2ch-bookmark-change-bookmark (bookmark-id)
-  "$B%V%C%/%^!<%/(B BOOKMARK-ID $B$N(B ID$B!"L>>N$rJQ99$9$k(B"
+  "$B%V%C%/%^!<%/(B BOOKMARK-ID $B$N(B ID$B!"L>>N$rJQ99$9$k!#(B"
   (let* ((bookmark (assoc bookmark-id navi2ch-bookmark-list))
 	 (id (navi2ch-read-string "new bookmark ID: " (car bookmark)))
 	 (name (navi2ch-read-string "new bookmark name: " (cadr bookmark))))
@@ -180,7 +184,7 @@ KEY $B$O(B (concat URI ARTID) ")
     (navi2ch-bookmark-save-info)))
 
 (defun navi2ch-bookmark-add-subr (bookmark-id board article)
-  "BOARD $B$H(B ARTICLE $B$GI=$5$l$k(B $B%9%l%C%I$rDI2C(B"
+  "BOARD $B$H(B ARTICLE $B$GI=$5$l$k(B $B%9%l%C%I$rDI2C!#(B"
   (unless (assoc bookmark-id navi2ch-bookmark-list)
     (navi2ch-bookmark-create-bookmark bookmark-id))
   (let ((bookmark (assoc bookmark-id navi2ch-bookmark-list)))
@@ -194,7 +198,7 @@ KEY $B$O(B (concat URI ARTID) ")
 
 (defun navi2ch-bookmark-add (bookmark-id board article)
   (navi2ch-bookmark-add-subr bookmark-id board article)
-  (message "Add current article to global bookmark."))
+  (message "Add current article to global bookmark"))
 
 (defun navi2ch-bookmark-insert-subject (num item)
   (navi2ch-bm-insert-subject
@@ -232,7 +236,7 @@ KEY $B$O(B (concat URI ARTID) ")
     (navi2ch-bookmark-delete-article (car node) board article)))
 
 (defun navi2ch-bookmark-delete-subr ()
-  "$B$=$N9T$r(B bookmark $B$+$i:o=|$9$k!#(B($B3NG'$J$7(B)"
+  "$B$=$N9T$r(B bookmark $B$+$i3NG'$J$7$G:o=|$9$k!#(B"
   (let ((item (save-excursion (beginning-of-line)
 			      (navi2ch-bookmark-get-property (point))))
 	(buffer-read-only nil))
@@ -249,14 +253,19 @@ KEY $B$O(B (concat URI ARTID) ")
 (defun navi2ch-bookmark-delete ()
   "$B$=$N9T$r(B bookmark $B$+$i:o=|$9$k!#(B"
   (interactive)
-  (if (y-or-n-p "Delete this line?")
+  (if (y-or-n-p "Delete this line? ")
       (navi2ch-bookmark-delete-subr)))
 
+(defun navi2ch-bookmark-delete-mark-article ()
+  (interactive)
+  (if (y-or-n-p "Delete these lines? ")
+      (navi2ch-bm-exec-subr 'navi2ch-bookmark-delete-subr)))
+
 (defun navi2ch-bookmark-copy (bookmark-id)
-  (interactive (list (navi2ch-bookmark-read-id "copy to: ")))
+  (interactive (list (navi2ch-bookmark-read-id "Copy to: ")))
   (if (equal bookmark-id
 	     navi2ch-bookmark-current-bookmark-id)
-      (message "Same bookmark.")
+      (message "Same bookmark")
     (save-excursion
       (beginning-of-line)
       (let ((item (navi2ch-bookmark-get-property (point))))
@@ -266,15 +275,15 @@ KEY $B$O(B (concat URI ARTID) ")
 	(navi2ch-bookmark-save-info)))))
 
 (defun navi2ch-bookmark-move (bookmark-id)
-  (interactive (list (navi2ch-bookmark-read-id "move to: ")))
+  (interactive (list (navi2ch-bookmark-read-id "Move to: ")))
   (if (equal bookmark-id
 	     navi2ch-bookmark-current-bookmark-id)
-      (message "Same bookmark.")
+      (message "Same bookmark")
     (navi2ch-bookmark-copy bookmark-id)
     (navi2ch-bookmark-delete-subr)))
 
 (defun navi2ch-bookmark-move-mark-article (bookmark-id)
-  (interactive (list (navi2ch-bookmark-read-id "move to: ")))
+  (interactive (list (navi2ch-bookmark-read-id "Move to: ")))
   (navi2ch-bm-exec-subr 'navi2ch-bookmark-move bookmark-id))
 
 (defun navi2ch-bookmark-cut ()
@@ -314,7 +323,7 @@ KEY $B$O(B (concat URI ARTID) ")
 	    (navi2ch-bookmark-insert-subject 0 (car pair)))
 	  (navi2ch-bm-renumber)
 	  (forward-line -1))
-      (message "stack is empty"))))
+      (message "Stack is empty"))))
 
 ;; (defun navi2ch-bookmark-exit ()
 ;;   (interactive)
@@ -322,7 +331,7 @@ KEY $B$O(B (concat URI ARTID) ")
 ;;   (set-window-configuration navi2ch-bookmark-window-configuration))
 
 (defun navi2ch-bookmark-goto-bookmark (bookmark-id)
-  (interactive (list (navi2ch-bookmark-read-id "bookmark id: ")))
+  (interactive (list (navi2ch-bookmark-read-id "Bookmark ID: ")))
   (let ((bookmark (assoc bookmark-id
 			 navi2ch-bookmark-list)))
     (navi2ch-list-select-board
@@ -331,7 +340,7 @@ KEY $B$O(B (concat URI ARTID) ")
 	   (cons 'type 'bookmark)))))
 
 (defun navi2ch-bookmark (board &rest args)
-  "bookmark $B$rI=<($9$k(B"
+  "bookmark $B$rI=<($9$k!#(B"
   (let ((bookmark-id (cdr (assq 'id board))))
     (navi2ch-bookmark-mode)
     (setq navi2ch-bookmark-current-bookmark-id bookmark-id)
@@ -408,7 +417,7 @@ KEY $B$O(B (concat URI ARTID) ")
                  ;; dat $B$r(B $BA4ItFI$^$:!"F,$N(B1$B9T$@$1FI$`$N$O$I$&$9$l$P(B?
 		 ;; $B26$bCN$i$J$$$G$9(B:-)
 		 (cdr (assq 'subject
-			    (navi2ch-article-get-first-message-from-file file)))))))
+			    (navi2ch-article-get-first-message-from-file file board)))))))
 	(when newsubject
 	  (setq article (navi2ch-put-alist 'subject newsubject article))
 	  (navi2ch-bookmark-add-subr
@@ -417,7 +426,44 @@ KEY $B$O(B (concat URI ARTID) ")
 
 (defun navi2ch-bookmark-fetch-mark-article (&optional force)
   (interactive "P")
-  (navi2ch-bm-exec-subr 'navi2ch-bookmark-fetch-article force))
+  (unless navi2ch-offline
+    (when navi2ch-bookmark-fetch-mark-article-check-update
+      (save-excursion
+	(let ((board-data-cache (navi2ch-make-cache nil 'equal)))
+	  (goto-char (point-min))
+	  (while (not (eobp))
+	    (navi2ch-bm-goto-mark-column)
+	    (when (looking-at "\\*")
+	      (let* ((item (navi2ch-bm-get-property-internal (point)))
+		     (board (navi2ch-bm-get-board-internal item))
+		     (article (navi2ch-bm-get-article-internal item))
+		     ;; (bbstype (cdr (assq 'bbstype board)))
+		     (board-uri (cdr (assq 'uri board)))
+		     (art-id (cdr (assq 'artid article)))
+		     (res (string-to-number
+			   (or (cdr (assq 'response
+					  (navi2ch-article-load-info board
+								     article)))
+			       "0")))
+		     new-res board-data)
+		(when (or (null navi2ch-bookmark-fetch-mark-article-no-check-regexp)
+			  (not (string-match navi2ch-bookmark-fetch-mark-article-no-check-regexp
+					     board-uri)))
+		  (setq board-data
+			(navi2ch-cache-get board-uri
+					   (mapcar
+					    (lambda (x)
+					      (cons (cdr (assq 'artid x))
+						    (string-to-number (cdr (assq 'response x)))))
+					    (navi2ch-board-get-updated-subject-list board))
+					   board-data-cache))
+		  (when (and res
+			     (setq new-res (cdr (assoc art-id board-data)))
+			     (navi2ch-bm-get-state)
+			     (<= new-res res))
+		    (navi2ch-bm-unmark)))))
+	    (forward-line)))))
+    (navi2ch-bm-exec-subr #'navi2ch-bookmark-fetch-article force)))
 
 (defun navi2ch-bookmark-change (changed-list)
   "$BJQ99$5$l$?HD$N(B bookmark $B$r=$@5$9$k!#(B
