@@ -1,6 +1,7 @@
-;;; navi2ch-message.el --- write message module for navi2ch
+;;; navi2ch-message.el --- write message module for navi2ch -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2000-2004 by Navi2ch Project
+;; Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2008 by
+;; Navi2ch Project
 
 ;; Author: Taiki SUGAWARA <taiki@users.sourceforge.net>
 ;; Keywords: network, 2ch
@@ -20,15 +21,25 @@
 ;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
+;;; Commentary:
+
+;;
+
 ;;; Code:
 (provide 'navi2ch-message)
 (defconst navi2ch-message-ident
-  "$Id: navi2ch-message.el,v 1.51 2004/05/02 15:12:50 nanashi Exp $")
+  "$Id$")
+(defconst navi2ch-message-samba24-sambatxt-url
+  "http://nullpo.s101.xrea.com/samba24/conv.xcg?browser=bbs2chreader&decsec=majority&offset=0&newline=crlf&output=download")
 
 (eval-when-compile (require 'cl))
 
 (require 'navi2ch)
 
+(eval-and-compile
+  (when (featurep 'xemacs)
+    (require 'timer)))
+
 (defvar navi2ch-message-aa-map nil)
 (unless navi2ch-message-aa-map
   (let ((map (make-sparse-keymap "Type ? for further options")))
@@ -83,11 +94,34 @@
     (bbstype . localfile)
     (uri . ,(concat "x-localbbs://" (navi2ch-expand-file-name "sendlog/")))))
 
+(defvar navi2ch-message-font-lock-keywords
+  `(("^>\\s-+.*$" . navi2ch-message-citation-face)
+    ("[>$B!d(B]+[0-9$B#0(B-$B#9(B]+" 0 navi2ch-message-link-face t)
+    (,navi2ch-article-url-regexp 0 navi2ch-message-url-face t)))
+
+(defvar navi2ch-message-link-face 'navi2ch-message-link-face)
+(defvar navi2ch-message-url-face 'navi2ch-message-url-face)
+(defvar navi2ch-message-citation-face 'navi2ch-message-citation-face)
+
+;; SAMBA24$B$N%G!<%?(B(url$B$@$NHDL>$@$N;~4V$@$N(B)
+(defvar navi2ch-message-samba24-samba-data nil)
+(defvar navi2ch-message-samba24-send-time nil
+  "$BHD(BID$B$H=q$-9~$_;~4V$rJ];}(B")
+;; $B%b!<%I%i%$%s$KI=<($9$k%+%&%s%H%@%&%s(B
+(defvar navi2ch-message-samba24-mode-string nil)
+(defvar navi2ch-message-samba24-show t
+  "non-nil $B$J$i=q$-9~$_5,@);~4V$rI=<((B.")
+;; $B8=:_I=<(Cf$N=q$-9~$_5,@);~4V%b!<%I%i%$%s(B
+(defvar navi2ch-message-samba24-mode-string nil)
+(defvar navi2ch-message-samba24-file-name "samba.txt"
+  "Samba24 $B$N5,@)IC?t>pJs$rJ];}$9$k%U%!%$%k$N%U%!%$%kL>(B.")
+(defvar navi2ch-message-samba24-update-timer nil)
+
 (defun navi2ch-message-write-message (board article &optional new sage)
   (when (or (not navi2ch-message-ask-before-write)
 	    (if (functionp navi2ch-message-ask-before-write)
-		(funcall navi2ch-message-ask-before-write "Write new message?")
-	      (y-or-n-p "Write new message?")))
+		(funcall navi2ch-message-ask-before-write "Write new message? ")
+	      (y-or-n-p "Write new message? ")))
     (if (and (get-buffer navi2ch-message-buffer-name)
 	     (or navi2ch-message-always-pop-message
 		 (not (navi2ch-message-kill-message))))
@@ -107,17 +141,20 @@
       (erase-buffer)
       (navi2ch-message-insert-header new sage)
       (setq navi2ch-mode-line-identification
-	    (if new
-		(format "*new message* [%s]"
-			(cdr (assq 'name navi2ch-message-current-board)))
-	      (format "Re: %s [%s]"
-		      (cdr (assq 'subject navi2ch-message-current-article))
-		      (cdr (assq 'name navi2ch-message-current-board)))))
+	    (navi2ch-message-make-mode-line-identification new))
       (navi2ch-set-mode-line-identification)
       (run-hooks 'navi2ch-message-setup-message-hook)
       (when sage
 	(run-hooks 'navi2ch-message-setup-sage-message-hook)))))
 
+(defun navi2ch-message-make-mode-line-identification (new)
+  (if new
+      (format "*new message* [%s]"
+	      (cdr (assq 'name navi2ch-message-current-board)))
+    (format "Re: %s [%s]"
+	    (cdr (assq 'subject navi2ch-message-current-article))
+	    (cdr (assq 'name navi2ch-message-current-board)))))
+
 (defun navi2ch-message-pop-message-buffer ()
   (interactive)
   (let ((buf (get-buffer navi2ch-message-buffer-name)))
@@ -137,20 +174,20 @@
   (when (get-buffer navi2ch-message-backup-buffer-name)
     (let ((inhibit-read-only t))
       (erase-buffer))
-    (insert-buffer navi2ch-message-backup-buffer-name)))
+    (insert-buffer-substring navi2ch-message-backup-buffer-name)))
 
 (defun navi2ch-message-insert-header (new sage)
   (and sage (setq sage "sage"))
   (when new
-    (insert "Subject: \n"))
-  (insert "From: "
+    (insert (navi2ch-read-only-string "Subject: ") "\n"))
+  (insert (navi2ch-read-only-string "From: ")
 	  (or (and navi2ch-message-remember-user-name
 		   (cdr (assq 'name navi2ch-message-current-article)))
 	      (cdr (assoc (cdr (assq 'id navi2ch-message-current-board))
 			  navi2ch-message-user-name-alist))
 	      navi2ch-message-user-name "")
 	  "\n"
-	  "Mail: "
+	  (navi2ch-read-only-string "Mail: ")
 	  (or sage
 	      (and navi2ch-message-remember-user-name
 		   (cdr (assq 'mail navi2ch-message-current-article)))
@@ -158,18 +195,18 @@
 			  navi2ch-message-mail-address-alist))
 	      navi2ch-message-mail-address "")
 	  "\n"
-	  (navi2ch-propertize navi2ch-message-header-separator
-			      'read-only t
-			      'front-sticky t 'rear-nonsticky t))
+	  (navi2ch-read-only-string
+	   (navi2ch-propertize navi2ch-message-header-separator
+			       'navi2ch-message-header-separator t)))
   (setq buffer-undo-list nil)
   (set-buffer-modified-p nil))
 
-(defsubst navi2ch-message-header-end ()
+(defun navi2ch-message-header-end ()
   (save-restriction
     (widen)
-    (if (get-text-property (point-min) 'read-only)
+    (if (get-text-property (point-min) 'navi2ch-message-header-separator)
 	(point-min)
-      (next-single-property-change (point-min) 'read-only))))
+      (next-single-property-change (point-min) 'navi2ch-message-header-separator))))
 
 (defun navi2ch-message-cleanup-message ()
   (save-excursion
@@ -188,11 +225,11 @@
 (defun navi2ch-message-send-message ()
   (interactive)
   (if navi2ch-offline
-      (message "Now offline.")
+      (message "Now offline")
     (when (or (not navi2ch-message-ask-before-send)
 	      (if (functionp navi2ch-message-ask-before-send)
-		  (funcall navi2ch-message-ask-before-send "Send message?")
-		(y-or-n-p "Send message?")))
+		  (funcall navi2ch-message-ask-before-send "Send message? ")
+		(y-or-n-p "Send message? ")))
       (widen)
       (run-hooks 'navi2ch-message-before-send-hook)
       (navi2ch-message-cleanup-message)
@@ -227,7 +264,7 @@
 	      (set-buffer (get-buffer-create
 			   navi2ch-message-backup-buffer-name))
 	      (erase-buffer)
-	      (insert-buffer buffer)))
+	      (insert-buffer-substring buffer)))
 	  (when navi2ch-message-trip
 	    (setq from (concat from "#" navi2ch-message-trip)))
 	  (let ((board navi2ch-message-current-board)
@@ -243,7 +280,7 @@
 	      (when navi2ch-message-save-sendlog
 		(navi2ch-message-add-sendlog from mail message subject
 					     board article))
-	      (message "waiting new message...")
+	      (message "Waiting new message...")
 	      (sleep-for navi2ch-message-wait-time)
 	      (message "%s%s" (current-message) "done")
 	      (save-excursion
@@ -256,6 +293,7 @@
 		    (navi2ch-article-sync navi2ch-message-force-sync)))))
 	    (when (get-buffer navi2ch-message-backup-buffer-name)
 	      (bury-buffer navi2ch-message-backup-buffer-name)))))
+      (navi2ch-message-samba24)
       (run-hooks 'navi2ch-message-after-send-hook)
       (navi2ch-message-exit 'after-send))))
 
@@ -284,7 +322,7 @@
 				 navi2ch-article-current-article))))))
 
 (defun navi2ch-message-cite-original (&optional arg)
-  "$B0zMQ$9$k(B"
+  "$B0zMQ$9$k!#(B"
   (interactive "P")
   (navi2ch-message-cite-original-from-number
    (save-excursion
@@ -294,7 +332,7 @@
 
 (defun navi2ch-message-cite-original-from-number (num &optional arg)
   "$BHV9f$rA*$s$G!"0zMQ$9$k!#(B"
-  (interactive "ninput number: \nP")
+  (interactive "nInput number: \nP")
   (when (< (point) (navi2ch-message-header-end))
     (error "Cannot cite in header"))
   (let (same msg board article)
@@ -335,8 +373,8 @@
   (when (or no-ask
 	    (not navi2ch-message-ask-before-kill)
 	    (if (functionp navi2ch-message-ask-before-kill)
-		(funcall navi2ch-message-ask-before-kill "Kill current message?")
-	      (y-or-n-p "Kill current message?")))
+		(funcall navi2ch-message-ask-before-kill "Kill current message? ")
+	      (y-or-n-p "Kill current message? ")))
     (kill-buffer navi2ch-message-buffer-name)
     t))
 
@@ -363,12 +401,16 @@
 	(fill-region-as-paragraph beg end arg)
 	t))))
 
-(defun navi2ch-message-mode ()
+(defun navi2ch-message-substitute-key-definitions ()
+  (dolist (old-new-def
+	   '((beginning-of-line . navi2ch-message-beginning-of-line)
+	     (back-to-indentation . navi2ch-message-back-to-indentation)))
+    (substitute-key-definition (car old-new-def) (cdr old-new-def)
+			       navi2ch-message-mode-map (current-global-map))))
+
+(define-derived-mode navi2ch-message-mode text-mode
+  "Navi2ch Message"
   "\\{navi2ch-message-mode-map}"
-  (interactive)
-  (kill-all-local-variables)
-  (setq major-mode 'navi2ch-message-mode)
-  (setq mode-name "Navi2ch Message")
   (set (make-local-variable 'fill-paragraph-function)
        'navi2ch-message-fill-paragraph)
   (set (make-local-variable 'paragraph-separate)
@@ -377,10 +419,10 @@
        navi2ch-message-paragraph-start)
   (set (make-local-variable 'auto-fill-inhibit-regexp)
        "^[A-Z][^: \n\t]+:")		; $B%X%C%@(B
-  (use-local-map navi2ch-message-mode-map)
+  (set (make-local-variable 'font-lock-defaults)
+       '(navi2ch-message-font-lock-keywords t))
   (navi2ch-message-setup-menu)
-  (run-hooks 'navi2ch-message-mode-hook)
-  (force-mode-line-update))
+  (navi2ch-message-substitute-key-definitions))
 
 (defun navi2ch-message-self-insert-aa ()
   "$B:G8eF~NO$7$?%-!<$K$7$?$,$C$F(B AA $B$rF~NO$9$k!#(B"
@@ -460,7 +502,7 @@
   "message buffer $B$,$"$k$H$-!"=q$-9~$_@h$N%9%l(B/$BHD$rI=<($7(B message buffer $B$K@Z$jBX$(!#(B"
   (interactive)
   (if (not (get-buffer navi2ch-message-buffer-name))
-      (message "No message buffer.")
+      (message "No message buffer")
     (delete-other-windows)
     (if navi2ch-message-current-article-buffer
 	;; $B4{B8%9%l$K=q$-9~$_(B $B"*(B $B=q$-9~$_@h$N%9%l$rI=<((B
@@ -482,6 +524,21 @@
     (other-window 1)
     (switch-to-buffer (get-buffer navi2ch-message-buffer-name))))
 
+(defun navi2ch-message-beginning-of-line (&optional n)
+  "$B9T$N@hF,$X0\F0!#(B
+header field $B$X0\F0$7$J$$0J30$O(B `beginning-of-line' $B$HF1$8!#(B"
+  (interactive "p")
+  (beginning-of-line n)
+  (when (< (point) (navi2ch-message-header-end))
+    (search-forward ": " nil t)))
+    
+(defun navi2ch-message-back-to-indentation ()
+  "$B9T$N:G=i$N6uGr$G$J$$2U=j$X0\F0!#(B
+header field $B$X0\F0$7$J$$0J30$O(B `back-to-indentation' $B$HF1$8!#(B"
+  (interactive)
+  (navi2ch-message-beginning-of-line)
+  (skip-chars-forward " \t"))
+
 ;; sendlog$B5!G=(B
 (defun navi2ch-message-sendlog-subject (board article)
   ;; $BAw?.$7$?%l%9$rJ]B8$9$k%9%l$N%?%$%H%k$rJV$9!#(B
@@ -517,7 +574,7 @@
     subject))
 
 (defun navi2ch-message-add-sendlog (from mail message subject board article)
-  (let ((navi2ch-localfile-default-file-modes ?\700)
+  (let ((navi2ch-localfile-default-file-modes (* 64 7))
 	;; $BAw?.95$((B $B$N%9%l%?%$$K(B &hearts; $B$H$+$r;H$($k$h$&$K!#(B
 	(navi2ch-decode-character-references nil)
 	(url (navi2ch-article-to-url board article))
@@ -528,7 +585,8 @@
 	(limit navi2ch-message-sendlog-response-limit)
 	larticle lsbj-list)
     (when (and lsubject lboard)
-      (setq message (format "Subject: %s\nURL: %s\n\n%s" sbj url message)
+      (setq message (funcall navi2ch-message-sendlog-message-format-function
+			     message sbj url board article)
 	    lsbj-list (navi2ch-board-get-updated-subject-list lboard)
 	    lsubject (navi2ch-message-sendlog-subject-with-volume
 		      lsubject fmt limit lsbj-list))
@@ -540,5 +598,138 @@
       (navi2ch-multibbs-send-message from mail message
 				     lsubject lboard larticle))))
 
+(defun navi2ch-message-sendlog-simple-message-format
+  (message subject url board article)
+  "$BAw?.95$($N%l%9$N%7%s%W%k$J%U%)!<%^%C%H!#(B"
+  (format "Subject: %s\nURL: %s\n\n%s" subject url message))
+
+(defun navi2ch-message-sendlog-message-format-with-board-name
+  (message subject url board article)
+  "$BAw?.95$($N%l%9$NHDL>IU$-$N%U%)!<%^%C%H!#(B"
+  (format "[%s]: %s\nURL: %s\n\n%s" (cdr (assq 'name board)) subject url message))
+
+;; TODO: 2ch $BFb$K$*$$$F$OHD(BID$B$,=EJ#$7$J$$$3$H$rA0Ds$H$7$F$$$k!#:#$N$H$3(B
+;; $B$m$OM-8z$@$,>-Mh$N$3$H$b9M$($k$HD>$9$Y$-!#(B
+
+(defun navi2ch-message-samba24-modeline ()
+  "$B=q$-9~$_7P2a;~4V$r%+%&%s%H%@%&%s$9$k(B."
+  (let* ((tmp-time (current-time))
+	 (now-time (+ (lsh (car tmp-time) 16) (nth 1 tmp-time)))
+	 samba-time time-diff)
+    (setq navi2ch-message-samba24-mode-string "")
+    (dolist (x navi2ch-message-samba24-send-time)
+      (let* ((id (car x))
+	     (id-normalized (if (string-match "^\\([^:]*\\):" id)
+				(match-string 1 id)
+			      id)))
+	(setq time-diff (- now-time (cdr x)))
+	(setq samba-time
+	      (navi2ch-message-samba24-search-samba 
+	       (navi2ch-message-samba24-board-conversion 'id id-normalized 'uri) 
+	       id-normalized))
+	(when samba-time
+	  (if (<= time-diff samba-time)
+	      (setq navi2ch-message-samba24-mode-string
+		    (format "%s:%d %s"
+			    (navi2ch-message-samba24-board-conversion 'id id 'name) 
+			    (- samba-time time-diff)
+			    navi2ch-message-samba24-mode-string))
+	    (setq navi2ch-message-samba24-send-time
+		  (delete x navi2ch-message-samba24-send-time))
+	    (unless navi2ch-message-samba24-send-time
+	      (cancel-timer navi2ch-message-samba24-update-timer)
+	      (setq navi2ch-message-samba24-update-timer nil))))))
+    (force-mode-line-update t)))
+
+(defun navi2ch-message-samba24 ()
+  "SAMBA24($BO"B3Ej9F5,@)(B)$B$NBP1~$N$?$a!"=q$-9~$_5v2DBT$A;~4V$rI=<($9$k!#(B
+$B%l%9Aw?.;~$K%3!<%k$5$l!"%b!<%I%i%$%s$G%+%&%s%H%@%&%s$rI=<($9$k(B"
+  (when navi2ch-message-samba24-show
+    (if (and (null navi2ch-message-samba24-samba-data)
+	     (null (navi2ch-message-samba24-read-samba)))
+	(message "samba.txt$B$,$"$j$^$;$s(B")
+      (let* ((tmp-time (current-time))
+	     (last-write-time (+ (lsh (car tmp-time) 16) (cadr tmp-time)))
+	     (id (cdr (assq 'id navi2ch-message-current-board)))
+	     (id-list (assoc id navi2ch-message-samba24-send-time))
+	     (id-normalized (if (string-match "^\\([^:]*\\):" id)
+				(match-string 1 id)
+			      id)))
+	(when (navi2ch-message-samba24-search-samba 
+	       (navi2ch-message-samba24-board-conversion 'id id-normalized 'uri)
+	       id-normalized)
+	  (when id-list
+	    (setq navi2ch-message-samba24-send-time
+		  (delete id-list navi2ch-message-samba24-send-time)))
+	  (setq navi2ch-message-samba24-send-time
+		(cons (cons id last-write-time)
+		      navi2ch-message-samba24-send-time))
+	  (setq navi2ch-message-samba24-update-timer
+		(or navi2ch-message-samba24-update-timer
+		    (run-at-time 1 1 'navi2ch-message-samba24-modeline))))))))
+
+(defun navi2ch-message-samba24-board-conversion (src val dst)
+  "$BHDL>!"(BID$B!"(BURL$B$J$I$NAj8_JQ49!#(B
+SRC=$BJQ4985$NO"A[%j%9%H:8B&(B VAL=$BJQ4985$NCM(B($B1&B&(B) DST=$BJQ49@h$N:8B&;XDj(B"
+  (catch 'loop
+    (dolist (x navi2ch-list-board-name-list)
+      (if (string= val (cdr (assq src x)))
+	  (throw 'loop (cdr (assq dst x)))))))
+
+(defun navi2ch-message-samba24-read-samba ()
+  "samba.txt $B$+$i3F%5!<%P!"HD$4$H$NO"B3Ej9F5,@);~4V$rFI$_9~$_!"%j%9%H$H$7$FJ];}$9$k(B.
+samba.txt $B$O(B http://nullpo.s101.xrea.com/samba24/ $B$+$i<hF@(B."
+  (interactive)
+  (let (navi2ch-message-samba24-file)
+    ;; $B:G?7$N(Bsamba.txt$B$r<hF@(B
+    (navi2ch-message-samba24-update)
+    (setq navi2ch-message-samba24-samba-data nil)
+    (setq navi2ch-message-samba24-file
+	  (navi2ch-expand-file-name navi2ch-message-samba24-file-name))
+    (when (and (file-exists-p navi2ch-message-samba24-file)
+	       (file-readable-p navi2ch-message-samba24-file))
+      (with-temp-buffer
+	(insert-file-contents navi2ch-message-samba24-file)
+	(goto-char (point-min))
+	(while (re-search-forward "\\([a-z0-9.]+\\)=\\([0-9]+\\)" nil t)
+	  (setq navi2ch-message-samba24-samba-data
+		(cons (cons (match-string 1)
+			    (string-to-number (match-string 2)))
+		      navi2ch-message-samba24-samba-data)))))
+    navi2ch-message-samba24-samba-data))
+
+;; FIXME: defsubst $B$K$7$?$$!#(B
+(defun navi2ch-message-samba24-search-samba (url id)
+  "$B%5!<%PL>!"HDL>$+$iO"B3Ej9F5,@);~4V$rF@$k(B."
+  (when (and (stringp url)
+	     (string-match "http://\\([^/]+\\)" url))
+    (or (cdr (assoc id navi2ch-message-samba24-samba-data))
+	(cdr (assoc (match-string 1 url) navi2ch-message-samba24-samba-data)))))
+
+(defun navi2ch-message-samba24-update ()
+  "samba24 $B$N5,@)>pJs$r99?7(B."
+  ;; $B%U%!%$%k$,F0E*@8@.$C$]$$$N$G(BIf-Modified-Since$B8+$J$$!)!J9bIi2Y!)!K(B
+  (navi2ch-net-update-file navi2ch-message-samba24-sambatxt-url
+			   (navi2ch-expand-file-name navi2ch-message-samba24-file-name)
+			   'file))
+
+(defun navi2ch-message-samba24-check (board)
+  "Samba24 $B$K$R$C$+$+$k$+$I$&$+%A%'%C%/(B."
+  (let* ((id (cdr (assq 'id board)))
+	 (last-write-time (cdr (assoc id
+				      navi2ch-message-samba24-send-time))))
+    (or (null last-write-time)
+	(let* ((samba-time (navi2ch-message-samba24-search-samba 
+			    (navi2ch-message-samba24-board-conversion 'id id 'uri) 
+			    id))
+	       (tmp-time (current-time))
+	       (cur-time (+ (lsh (car tmp-time) 16) (cadr tmp-time)))
+	       (diff-time (- (+ last-write-time samba-time)
+			     cur-time)))
+	  (or (<= diff-time 0)
+	      (yes-or-no-p (format 
+			    "$B$"$H(B %d $BICBT$C$?$[$&$,$$$$$H;W$&$1$I!"K\Ev$K=q$-$3$`(B? "
+			    diff-time)))))))
+
 (run-hooks 'navi2ch-message-load-hook)
 ;;; navi2ch-message.el ends here
