@@ -1,6 +1,6 @@
-;;; navi2ch-multibbs.el --- View 2ch like BBS module for Navi2ch.
+;;; navi2ch-multibbs.el --- View 2ch like BBS module for Navi2ch. -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2002-2004 by Navi2ch Project
+;; Copyright (C) 2002, 2003, 2004, 2005, 2006, 2008 by Navi2ch Project
 
 ;; Author:
 ;; Part5 $B%9%l$N(B 509 $B$NL>L5$7$5$s(B
@@ -30,36 +30,39 @@
 ;;; Code:
 (provide 'navi2ch-multibbs)
 (defconst navi2ch-multibbs-ident
-  "$Id: navi2ch-multibbs.el,v 1.33 2004/05/05 09:03:07 nanashi Exp $")
+  "$Id$")
 
+(eval-when-compile (require 'cl))
 (require 'navi2ch-http-date)
 (require 'navi2ch)
+(require 'navi2ch-be2ch)
 
-(defvar navi2ch-multibbs-func-alist nil
-  "BBS $B$N<oN`$H4X?t72$N(B alist$B!#(B
-$B3FMWAG$O(B
-\(BBSTYPE . FUNC-ALIST)
+(defvar navi2ch-multibbs-func-table nil
+  "BBS $B$N<oN`$H4X?t72$N(B hash$B!#(B
+BBSTYPE $B$r(B key $B$K(B FUNC-TABLE $B$,3JG<$5$l$k!#(B
 BBSTYPE: BBS $B$N<oN`$rI=$9%7%s%\%k!#(B
-FUNC-ALIST: $B$=$N(B BBS $B$G$NF0:n$r;XDj$9$k4X?t72!#(B
+FUNC-TABLE: $B$=$N(B BBS $B$G$NF0:n$r;XDj$9$k4X?t72!#(B
 
-FUNC-ALIST $B$O0J2<$NDL$j(B
-\((bbs-p			. BBS-P-FUNC)
- (subject-callback	. SUBJECT-CALLBACK-FUNC)
- (article-update	. ARTICLE-UPDATE-FUNC)
- (article-to-url	. ARTICLE-TO-URL-FUNC)
- (url-to-board		. URL-TO-BOARD-FUNC)
- (url-to-article	. URL-TO-ARTICLE-FUNC)
- (send-message		. SEND-MESSAGE-FUNC)
- (send-success-p	. SEND-MESSAGE-SUCCESS-P-FUNC)
- (error-string		. ERROR-STRING-FUNC)
- (board-update		. BOARD-UPDATE-FUNC)
- (board-get-file-name	. BOARD-GET-FILE-NAME-FUNC))
+FUNC-TABLE $B$O0J2<$N:8B&$N%7%s%\%k$r(B key $B$K(B
+$B4X?t$,3JG<$5$l$k!#(B
+bbs-p			BBS-P-FUNC
+subject-callback	SUBJECT-CALLBACK-FUNC
+article-update	ARTICLE-UPDATE-FUNC
+article-to-url	ARTICLE-TO-URL-FUNC
+url-to-board		URL-TO-BOARD-FUNC
+url-to-article	URL-TO-ARTICLE-FUNC
+send-message		SEND-MESSAGE-FUNC
+extract-post          EXTRACT-POST-FUNC
+send-success-p	SEND-MESSAGE-SUCCESS-P-FUNC
+error-string		ERROR-STRING-FUNC
+board-update		BOARD-UPDATE-FUNC
+board-get-file-name	BOARD-GET-FILE-NAME-FUNC
 
 BBS-P-FUNC(URI):
     URI $B$,$=$N(B BBS $B$N$b$N$J$i$P(B non-nil $B$rJV$9!#(B
 
 SUBJECT-CALLBACK-FUNC():
-    subject.txt $B$r<hF@$9$k$H$-$K(B navi2ch-net-update-file $B$G;H$o$l$k%3!<(B
+    subject.txt $B$r<hF@$9$k$H$-$K(B `navi2ch-net-update-file' $B$G;H$o$l$k%3!<(B
     $B%k%P%C%/4X?t(B
 
 ARTICLE-UPDATE-FUNC(BOARD ARTICLE START):
@@ -77,9 +80,19 @@ URL-TO-ARTICLE-FUNC(URL):
 URL $B$+$i(B article $B$KJQ49$9$k!#(B
 
 SEND-MESSAGE-FUNC(FROM MAIL MESSAGE
-		  SUBJECT BBS KEY TIME BOARD ARTICLE):
+		  SUBJECT BBS KEY TIME BOARD ARTICLE POST):
     MESSAGE $B$rAw?.$9$k!#(B
 
+EXTRACT-POST-FUNC(OLD-POST BUFFER):
+    MESSAGE $B$N:FAw?.$K;H$&>pJs$r<h$j=P$9!#<h$j=P$7$?%G!<%?$r(B
+    $BJV$jCM$H$7$FJV$9$H!":FAw$N$?$a$K(BSEND-MESSAGE-FUNC$B$r8F$S=P$9(B
+    $B$H$-$K!"$=$NCM$,(BPOST$B0z?t$KB+G{$5$l$^$9!#(B
+     
+    BUFFER$B$K$O!"(BSEND-MESSAGE-FUNC$B$,JV$7$?(BPROC$B$+$i<h$j=P$7$?%3%s(B
+    $B%F%s%D$r%G%3!<%I$7$?$b$N$,A^F~$5$l$F$$$^$9!#:FAw$r7+$jJV$9(B
+    $B>l9g!":G8e$K(BEXTRACT-POST-FUNC$B$,JV$7$?CM$,(BOLD-POST$B$KB+G{$5$l(B
+    $B$F$$$^$9!#(B
+
 SEND-MESSAGE-SUCCESS-P-FUNC(PROC):
     PROC $B$NAw?.%;%C%7%g%s$,@.8y$7$F$$$l$P(B non-nil $B$r!"(B
     $B<:GT$7$?$i(B nil $B$r!":F;n9T2DG=$J<:GT$J$i(B 'retry $B$rJV$9!#(B
@@ -92,8 +105,7 @@ BOARD-UPDATE-FUNC(BOARD):
 
 BOARD-GET-FILE-NAME-FUNC(BOARD &optional FILE-NAME)
     BOARD $B$N>pJs$rJ]B8$9$k%G%#%l%/%H%j$r4p=`$H$7$F!"(BFILE-NAME $B$N(B
-    $B@dBP%Q%9$rJV$9!#(B
-")
+    $B@dBP%Q%9$rJV$9!#(B")
 
 (defvar navi2ch-multibbs-variable-alist nil
   "BBS $B$N<oN`$HJQ?t72$N(B alist$B!#(B
@@ -106,24 +118,26 @@ VARIABLE-ALIST $B$O0J2<$NDL$j(B
 \((coding-system		. CODING-SYSTEM-VAR))
 
 CODING-SYSTEM-VAR:
-    $B$=$N(B BBS $B$N%U%!%$%k$NJ8;z%3!<%I(B
-")
+    $B$=$N(B BBS $B$N%U%!%$%k$NJ8;z%3!<%I(B")
 
+(defvar navi2ch-2ch-board-file-name-cache nil)
+(defvar navi2ch-2ch-board-file-name-cache-limit 1000)
 
-(defun navi2ch-multibbs-get-bbstype-subr (uri list)
-  (if list
-      (let ((bbstype    (caar list))
-	    (func       (cdr (assq 'bbs-p (cdar list)))))
-	(if (and func (funcall func uri))
-	    bbstype
-	  (navi2ch-multibbs-get-bbstype-subr uri (cdr list))))))
+(defun navi2ch-multibbs-get-bbstype-subr (uri table)
+  (when (hash-table-p table)
+    (catch 'loop
+      (maphash (lambda (type func-table)
+		 (let ((func (gethash 'bbs-p func-table)))
+		   (when (and func (funcall func uri))
+		     (throw 'loop type))))
+	       table))))
 
 (defun navi2ch-multibbs-set-bbstype (board type)
   (when (consp board)
     (setcdr board
 	    (cons (cons 'bbstype type) (cdr board)))))
 
-(defun navi2ch-multibbs-get-bbstype (board)
+(defsubst navi2ch-multibbs-get-bbstype (board)
   (let ((type (cdr (assq 'bbstype board))))
     (unless type
       (setq type (navi2ch-multibbs-url-to-bbstype
@@ -131,19 +145,25 @@ CODING-SYSTEM-VAR:
       (navi2ch-multibbs-set-bbstype board type))
     type))
 
+(defsubst navi2ch-multibbs-get-func (bbstype func &optional default-func)
+  (let ((func-table (gethash bbstype navi2ch-multibbs-func-table)))
+    (or (and func-table
+	     (gethash func func-table))
+	default-func)))
+
 (defun navi2ch-multibbs-subject-callback (board)
   (navi2ch-multibbs-get-func
    (navi2ch-multibbs-get-bbstype board)
    'subject-callback 'navi2ch-2ch-subject-callback))
 
 (defmacro navi2ch-multibbs-defcallback (name spec &rest body)
-  "navi2ch-net-update-file $B$KEO$9(B callback $B$rDj5A$9$k!#(B
+  "`navi2ch-net-update-file' $B$KEO$9(B callback $B$rDj5A$9$k!#(B
 SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
 $B<B:]$K$O!"(Bcallback $B$rDj5A$9$k$N$KI,MW$J(B BBSTYPE $B$JHD$N(B coding-system
 $B$K$h$k(B decode, encode $B=hM}$r(B BODY $B$rI>2A$9$kA08e$K9T$J$&!"(BNAME $B$H$$$&(B
 $B0z?t(B [ARG]... $B$r;}$D4X?t$,Dj5A$5$l$k!#(B"
-  (let ((bbstype (gensym "--bbstype--"))
-	(decoding (gensym "--decoding--"))
+  (let ((bbstype (make-symbol "--bbstype--"))
+	(decoding (make-symbol "--decoding--"))
 	docstring)
     (when (stringp (car body))
       (setq docstring (car body))
@@ -156,8 +176,9 @@ SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
 	      (,decoding (navi2ch-multibbs-get-variable
 			  ,bbstype 'coding-system
 			  navi2ch-coding-system)))
-	 (decode-coding-region (point-min) (point-max)
-			       ,decoding)
+	 (navi2ch-ifxemacs
+	     (navi2ch-decode-coding-region-linewise (point-min) (point-max) ,decoding)
+	   (decode-coding-region (point-min) (point-max) ,decoding))
 	 (navi2ch-set-buffer-multibyte t)
 	 ,@body
 	 (encode-coding-region (point-min) (point-max)
@@ -172,9 +193,12 @@ SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
     (funcall func board article start)))
 
 (defun navi2ch-multibbs-regist (bbstype func-alist variable-alist)
-  (setq navi2ch-multibbs-func-alist
-	(cons (cons bbstype func-alist)
-	      navi2ch-multibbs-func-alist))
+  (unless navi2ch-multibbs-func-table
+    (setq navi2ch-multibbs-func-table
+	  (make-hash-table :size 6))) ;FIXME: 6 $B$G$?$j$k$H;W$&$1$I$I$&$9$+$M!#(B
+  (puthash bbstype
+	   (navi2ch-alist-to-hash func-alist)
+	   navi2ch-multibbs-func-table)
   (setq navi2ch-multibbs-variable-alist
 	(cons (cons bbstype variable-alist)
 	      navi2ch-multibbs-variable-alist)))
@@ -185,12 +209,6 @@ SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
    (navi2ch-multibbs-get-bbstype board)
    func default-func))
 
-(defun navi2ch-multibbs-get-func (bbstype func &optional default-func)
-  (or (cdr (assq func
-		 (cdr (assq bbstype
-			    navi2ch-multibbs-func-alist))))
-      default-func))
-
 (defun navi2ch-multibbs-get-variable
   (bbstype variable &optional default-value)
   (or (cdr (assq variable
@@ -201,7 +219,7 @@ SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
 (defun navi2ch-multibbs-url-to-bbstype (url)
   (or
    (and url
-	(navi2ch-multibbs-get-bbstype-subr url navi2ch-multibbs-func-alist))
+	(navi2ch-multibbs-get-bbstype-subr url navi2ch-multibbs-func-table))
    'unknown))
 
 (defun navi2ch-multibbs-url-to-article (url)
@@ -236,19 +254,27 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 	      h (- h 0)))
       (cons h l))))
 
-(defvar navi2ch-multibbs-send-message-retry-comfirm-function #'yes-or-no-p)
+(defun navi2ch-multibbs-send-message-error-string (board proc)
+  (let* ((func (navi2ch-multibbs-get-func
+		(navi2ch-multibbs-get-bbstype board)
+		'error-string
+		'navi2ch-2ch-send-message-error-string))
+	 (err (funcall func proc)))
+    (or err
+	(let ((status (and proc (navi2ch-net-get-status proc))))
+	  (when status
+	    (concat "HTTP status: " status))))))
 
 (defun navi2ch-multibbs-send-message
   (from mail message subject board article)
   (let* ((bbstype      (navi2ch-multibbs-get-bbstype board))
 	 (send         (navi2ch-multibbs-get-func
 			bbstype 'send-message 'navi2ch-2ch-send-message))
+	 (extract-post (navi2ch-multibbs-get-func
+			bbstype 'extract-post 'navi2ch-2ch-extract-post))
 	 (success-p    (navi2ch-multibbs-get-func
 			bbstype 'send-success-p
 			'navi2ch-2ch-send-message-success-p))
-	 (error-string (navi2ch-multibbs-get-func
-			bbstype 'error-string
-			'navi2ch-2ch-send-message-error-string))
 	 (bbs          (let ((uri (navi2ch-board-get-uri board)))
 			 (string-match "\\([^/]+\\)/$" uri)
 			 (match-string 1 uri)))
@@ -266,10 +292,11 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 					    navi2ch-net-http-proxy-password))
 	 (tries 2)			; $BAw?.;n9T$N:GBg2s?t(B
 	 (message-str "send message...")
-	 (result 'retry))
+	 (result 'retry)
+	 (post-data nil))
     (dotimes (i tries)
       (let ((proc (funcall send from mail message subject bbs key time
-			   board article)))
+			   board article post-data)))
 	(message message-str)
 	(setq result (funcall success-p proc))
 	(cond ((eq result 'retry)
@@ -277,17 +304,15 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 		 (with-temp-buffer
 		   (insert (decode-coding-string
 			    (navi2ch-net-get-content proc)
-			    navi2ch-coding-system))
+			    (navi2ch-board-get-coding-system board)))
+		   (setq post-data (funcall extract-post post-data (current-buffer)))
 		   (navi2ch-replace-html-tag-with-buffer)
 		   (goto-char (point-min))
 		   (while (re-search-forward "[ \t]*\n\\([ \t]*\n\\)*" nil t)
 		     (replace-match "\n"))
 		   (delete-other-windows)
 		   (switch-to-buffer (current-buffer))
-		   (unless (funcall (or navi2ch-multibbs-send-message-retry-comfirm-function
-					#'yes-or-no-p)
-				    "Retry? ")
-		     (navi2ch-board-save-spid board nil)
+		   (unless (y-or-n-p "Retry? ")
 		     (return nil))))
 	       (sit-for navi2ch-message-retry-wait-time)
 	       (setq message-str "re-send message..."))
@@ -295,25 +320,25 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 	       (message (concat message-str "succeed"))
 	       (return result))
 	      (t
-	       (let ((err (funcall error-string proc)))
+	       (let ((err (navi2ch-multibbs-send-message-error-string board proc)))
 		 (if (stringp err)
 		     (message (concat message-str "failed: %s") err)
 		   (message (concat message-str "failed"))))
 	       (return nil)))))))
 
-(defun navi2ch-multibbs-board-update (board)
+(defsubst navi2ch-multibbs-board-update (board)
   (let ((func (navi2ch-multibbs-get-func-from-board
 	       board 'board-update 'navi2ch-2ch-board-update)))
     (funcall func board)))
 
-(defun navi2ch-multibbs-board-get-file-name (board &optional file-name)
+(defsubst navi2ch-multibbs-board-get-file-name (board &optional file-name)
   (let ((func (navi2ch-multibbs-get-func-from-board
 	       board 'board-get-file-name 'navi2ch-2ch-board-get-file-name)))
     (funcall func board file-name)))
 
 ;;;-----------------------------------------------
 
-(defun navi2ch-2ch-subject-callback ()
+(defsubst navi2ch-2ch-subject-callback ()
   (when navi2ch-board-use-subback-html
     (navi2ch-board-make-subject-txt)))
 
@@ -324,19 +349,11 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
   (let ((file (navi2ch-article-get-file-name board article))
 	(time (cdr (assq 'time article)))
 	url header kako-p)
-    (if (and (navi2ch-enable-readcgi-p
-	      (navi2ch-board-get-host board)))
-	(progn
-	  (setq url (navi2ch-article-get-readcgi-raw-url board article start))
-	  (setq header (navi2ch-net-update-file-with-readcgi url file
-							     time start))
-	  ;; read.cgi $B$r;H$C$F$F%(%i!<$K$J$k$N$O%[%s%H$K%(%i!<$N$H$-(B
-	  (setq kako-p (navi2ch-net-get-state 'kako header)))
-      (setq url (navi2ch-article-get-url board article))
-      (setq header (if start
-		       (navi2ch-net-update-file-diff url file time)
-		     (navi2ch-net-update-file url file time)))
-      (setq kako-p (navi2ch-net-get-state 'error header)))
+    (setq url (navi2ch-article-get-url board article))
+    (setq header (if start
+		     (navi2ch-net-update-file-diff url file time)
+		   (navi2ch-net-update-file url file time)))
+    (setq kako-p (navi2ch-net-get-state 'error header))
     (when kako-p
       (setq url (navi2ch-article-get-kako-url board article))
       (setq header (navi2ch-net-update-file url file))
@@ -399,81 +416,124 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
 	  (setq list (cons (cons 'kako kako) list)))
 	list))))
 
+(defvar navi2ch-2ch-send-message-last-board nil)
+
 (defun navi2ch-2ch-send-message
-  (from mail message subject bbs key time board article)
-  (let ((url         (navi2ch-board-get-bbscgi-url board))
-	(referer     (navi2ch-board-get-uri board))
-	(spid        (navi2ch-board-load-spid board))
-	(param-alist (list
-		      (cons "submit" "$B=q$-9~$`(B")
-		      (cons "FROM"   (or from ""))
-		      (cons "mail"   (or mail ""))
-		      (cons "bbs"    bbs)
-		      (cons "time"   time)
-		      (cons "MESSAGE" message)
-		      (if subject
-			  (cons "subject" subject)
-			(cons "key"    key)))))
-    (setq spid
-	  (when (and (consp spid)
-		     (navi2ch-compare-times (cdr spid) (current-time)))
-	    (car spid)))
-    (let ((proc
-	   (navi2ch-net-send-request
-	    url "POST"
-	    (list (cons "Content-Type" "application/x-www-form-urlencoded")
-		  (cons "Cookie" (concat "NAME=" from "; MAIL=" mail
-					 (if spid (concat "; SPID=" spid
-							  "; PON=" spid))))
-		  (cons "Referer" referer))
-	    (navi2ch-net-get-param-string param-alist))))
-      (setq spid (navi2ch-net-send-message-get-spid proc))
-      (if spid (navi2ch-board-save-spid board spid))
-      proc)))
+  (from mail message subject bbs key time board article &optional post)
+  (when (navi2ch-message-samba24-check board)
+    (let* ((url         (navi2ch-board-get-bbscgi-url board))
+	   (referer     (navi2ch-board-get-uri board))
+	   (param-alist (list
+			 (cons "submit" "$B=q$-9~$`(B")
+			 (cons "FROM"   (or from ""))
+			 (cons "mail"   (or mail ""))
+			 (cons "bbs"    bbs)
+			 (cons "time"   time)
+			 (cons "MESSAGE" message)
+			 (if subject
+			     (cons "subject" subject)
+			   (cons "key"    key))))
+	   (coding-system (navi2ch-board-get-coding-system board))
+	   (cookies (navi2ch-net-match-cookies url)))
+      (dolist (param post)
+	(unless (assoc (car param) param-alist)
+	  (push param param-alist)))
+      (setq navi2ch-2ch-send-message-last-board board)
+      (let ((proc
+	     (navi2ch-net-send-request
+	      url "POST"
+	      (list (cons "Content-Type" "application/x-www-form-urlencoded")
+		    (cons "Cookie"
+			  (navi2ch-net-cookie-string cookies coding-system))
+		    (cons "Referer" referer))
+	      (navi2ch-net-get-param-string param-alist
+					    coding-system))))
+	(navi2ch-net-update-cookies url proc coding-system)
+	(navi2ch-net-save-cookies)
+	proc))))
 
 (defun navi2ch-2ch-article-to-url
   (board article &optional start end nofirst)
   "BOARD, ARTICLE $B$+$i(B url $B$KJQ49!#(B
 START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
-  (let ((url (navi2ch-board-get-readcgi-url board)))
-    (setq url (concat url (cdr (assq 'artid article)) "/"))
-    (if (numberp start)
-	(setq start (number-to-string start)))
-    (if (numberp end)
-	(setq end (number-to-string end)))
-    (if (equal start end)
-	(concat url start)
-      (concat url
-	      start (and (or start end) "-") end
-	      (and nofirst "n")))))
+  (let ((uri (navi2ch-board-get-uri board))
+	(start (if (numberp start)
+		   (number-to-string start)
+		 start))
+	(end (if (numberp end)
+		 (number-to-string end)
+	       end)))
+    (if (string-match "\\(.+\\)/\\([^/]+\\)/$" uri)
+	(format "%s/test/read.cgi/%s/%s/%s"
+		(match-string 1 uri) (match-string 2 uri)
+		(cdr (assq 'artid article))
+		(if (equal start end)
+		    (or start "")
+		  (concat start (and (or start end) "-") end
+			  (and nofirst "n")))))))
 
-(defalias 'navi2ch-2ch-send-message-success-p
-  'navi2ch-net-send-message-success-p)
-(defalias 'navi2ch-2ch-send-message-error-string
-  'navi2ch-net-send-message-error-string)
+(defun navi2ch-2ch-send-message-success-p (proc)
+  (navi2ch-net-send-message-success-p
+   proc
+   (navi2ch-board-get-coding-system
+    navi2ch-2ch-send-message-last-board)))
+
+(defun navi2ch-2ch-send-message-error-string (proc)
+  (navi2ch-net-send-message-error-string
+   proc
+   (navi2ch-board-get-coding-system
+    navi2ch-2ch-send-message-last-board)))
 
 (defun navi2ch-2ch-board-update (board)
   (let ((file (navi2ch-board-get-file-name board))
 	(time (cdr (assq 'time board))))
-    (if navi2ch-board-enable-readcgi
-	(navi2ch-net-update-file-with-readcgi
-	 (navi2ch-board-get-readcgi-raw-url board) file time)
-      (let ((url (navi2ch-board-get-url
-		  board (if navi2ch-board-use-subback-html
-			    navi2ch-board-subback-file-name)))
-	    (func (navi2ch-multibbs-subject-callback board)))
-	(navi2ch-net-update-file url file time func)))))
+    (let ((url (navi2ch-board-get-url
+		board (if navi2ch-board-use-subback-html
+			  navi2ch-board-subback-file-name)))
+	  (func (navi2ch-multibbs-subject-callback board)))
+      (navi2ch-net-update-file url file time func))))
 
 (defun navi2ch-2ch-board-get-file-name (board &optional file-name)
-  (let ((uri (navi2ch-board-get-uri board)))
+  (let ((uri (navi2ch-board-get-uri board))
+	(file-name (or file-name
+		       navi2ch-board-subject-file-name)))
     (when uri
-      (cond ((string-match "http://\\(.+\\)" uri)
-	     (navi2ch-expand-file-name
-	      (concat (match-string 1 uri)
-		      (or file-name navi2ch-board-subject-file-name))))
-	    ((string-match "file://\\(.+\\)" uri)
-	     (expand-file-name (or file-name
-				   navi2ch-board-subject-file-name)
-			       (match-string 1 uri)))))))
+      (or navi2ch-2ch-board-file-name-cache
+	  (setq navi2ch-2ch-board-file-name-cache
+		(navi2ch-make-cache navi2ch-2ch-board-file-name-cache-limit
+				    'equal)))
+      (navi2ch-cache-get
+       (cons uri file-name)
+       (cond ((string-match "http://\\(.+\\)" uri)
+	      (navi2ch-expand-file-name
+	       (concat (match-string 1 uri)
+		       file-name)))
+	     ((string-match "file://\\(.+\\)" uri)
+	      (expand-file-name file-name
+				(match-string 1 uri))))
+       navi2ch-2ch-board-file-name-cache))))
+
+(defun navi2ch-2ch-extract-post (old-post buffer)
+  ;; Get hana and mogera from following string.
+  ;; <input type=hidden name="hana" value="mogera">  
+  (with-current-buffer buffer
+    (save-excursion
+      (save-match-data
+	(goto-char (point-min))
+	(let ((case-fold-search t)
+	      (re "\\<%s=\\(\"\\([^\"]*\\)\"\\|[^\"> \r\n\t]*\\)")
+	      r)
+	  (while (re-search-forward "<input\\>[^>]+>" nil t)
+	    (let ((str (match-string 0)) name value)
+	      (and (string-match (format re "name") str)
+		   (setq name (or (match-string 2 str)
+				  (match-string 1 str)))
+		   (string-match (format re "value") str)
+		   (setq value (or (match-string 2 str)
+				   (match-string 1 str)))
+		   (setq name (navi2ch-replace-html-tag name)
+			 value (navi2ch-replace-html-tag value))
+		   (push (cons name value) r))))
+	  (nreverse r))))))
 
 ;;; navi2ch-multibbs.el ends here
