@@ -29,7 +29,7 @@
 (provide 'navi2ch-oyster)
 
 (defconst navi2ch-oyster-ident
-  "$Id: navi2ch-oyster.el,v 1.30.2.3 2008/08/26 14:08:08 nawota Exp $")
+  "$Id$")
 
 (require 'navi2ch-util)
 (require 'navi2ch-multibbs)
@@ -205,11 +205,11 @@ DIFF $B$,(B non-nil $B$J$i$P:9J,$r<hF@$9$k!#(B
                      (not (search-forward "HTTP/1\\.[01] \\([0-9]+\\)")))
            (accept-process-output proc)
            (message "Retrying")
-           (sit-for 3))
+           (sleep-for 3))
          (let ((i 10))
            (catch 'loop
              (while (>= (setq i (1- i)) 0)
-               (sit-for 1)	  ; $B2?$@$+$&$^$/F0$+$J$$$N$G(Bwait$BF~$l$?(B
+               (sleep-for 1)	  ; $B2?$@$+$&$^$/F0$+$J$$$N$G(Bwait$BF~$l$?(B
                (goto-char (point-min))
                ;; $B:G8e$^$G8+$D$+$i$J$$$^$^$@$H%(%i!<(B
                (when (search-forward "SESSION-ID=" nil (> i 0))
