@@ -28,7 +28,7 @@
 ;;; Code:
 (provide 'navi2ch)
 (defconst navi2ch-ident
-  "$Id: navi2ch.el,v 1.92.2.1 2008/08/26 14:07:59 nawota Exp $")
+  "$Id$")
 
 (eval-when-compile (require 'cl))
 
@@ -390,9 +390,12 @@ DISPLAY $B$,(B `article' $B$N$H$-$O(B article $B$rI=<($9$kMQ$KJ,3d$9$k!#(B
 
 (defun navi2ch-goto-url (url &optional force)
   "URL $B$+$i%9%l$^$?$OHD$rA*$V!#(B"
-  (interactive (list (navi2ch-read-string "URL: "
-					  (get-text-property (point) 'url))
-		     current-prefix-arg))
+  (interactive (list (navi2ch-read-string 
+		      "URL: "
+		      (when (eq (get-text-property (point) 'navi2ch-link-type)
+				'url)
+			(get-text-property (point) 'navi2ch-link))
+		      current-prefix-arg)))
   (let ((article (navi2ch-article-url-to-article url))
 	(board (navi2ch-board-url-to-board url)))
     (cond (article
